@@ -6,6 +6,7 @@
 #include <stdlib.h>
 
 #include <orc/orcprogram.h>
+#include <orc/orcdebug.h>
 
 /**
  * SECTION:orc
@@ -30,9 +31,12 @@ orc_init (void)
 
   _inited = 1;
 
+  ORC_ASSERT(sizeof(OrcExecutor) == sizeof(OrcExecutorAlt));
+
   _orc_debug_init();
   orc_opcode_init();
   orc_c_init();
+  orc_c64x_c_init();
   orc_mmx_init();
   orc_sse_init();
   orc_powerpc_init();
