@@ -5,8 +5,12 @@
 #include <string.h>
 #include <stdlib.h>
 
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #ifdef HAVE_CODEMEM_MMAP
 #include <sys/mman.h>
 #endif
@@ -14,6 +18,7 @@
 #include <orc/orcprogram.h>
 #include <orc/orcdebug.h>
 
+
 #define SIZE 65536
 
 
@@ -21,17 +26,38 @@
 void
 orc_compiler_allocate_codemem (OrcCompiler *compiler)
 {
-  char filename[32] = "/tmp/orcexecXXXXXX";
   int fd;
   int n;
 
-  fd = mkstemp (filename);
-  if (fd == -1) {
-    /* FIXME oh crap */
-    ORC_COMPILER_ERROR (compiler, "failed to create temp file");
-    return;
+  /* FIXME something combining these two would be ideal */
+#if 1
+  {
+    char filename[32] = "/tmp/orcexecXXXXXX";
+    fd = mkstemp (filename);
+    if (fd == -1) {
+      /* FIXME oh crap */
+      ORC_COMPILER_ERROR (compiler, "failed to create temp file");
+      return;
+    }
+    unlink (filename);
   }
-  unlink (filename);
+#else
+  {
+    char *filename;
+
+    filename = malloc (strlen ("/tmp/orcexec") +
+        strlen (compiler->program->name) + 1);
+    sprintf(filename, "/tmp/orcexec%s", compiler->program->name);
+    fd = open (filename, O_RDWR | O_CREAT, S_IRWXU);
+    if (fd == -1) {
+      /* FIXME oh crap */
+      ORC_COMPILER_ERROR (compiler, "failed to create temp file");
+      return;
+    }
+    unlink (filename);
+    free (filename);
+  }
+#endif
 
   n = ftruncate (fd, SIZE);
 
