@@ -4,6 +4,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <math.h>
 
 #include <orc/orcprogram.h>
 #include <orc/orcdebug.h>
@@ -87,6 +88,25 @@ orc_target_get_default_flags (OrcTarget *target)
   return target->get_default_flags();
 }
 
+const char *
+orc_target_get_preamble (OrcTarget *target)
+{
+  if (target->get_asm_preamble == NULL) return "";
+
+  return target->get_asm_preamble ();
+}
+
+const char *
+orc_target_get_asm_preamble (const char *target)
+{
+  OrcTarget *t;
+
+  t = orc_target_get_by_name (target);
+  if (t == NULL) return "";
+
+  return orc_target_get_preamble (t);
+}
+
 #if 0
 int
 orc_opcode_get_list (OrcOpcode **list)
@@ -131,7 +151,7 @@ orc_rule_set_new (OrcOpcodeSet *opcode_set, OrcTarget *target,
 
   memset (rule_set, 0, sizeof(OrcRuleSet));
 
-  rule_set->opcode_set = opcode_set;
+  rule_set->opcode_major = opcode_set->opcode_major;
   rule_set->required_target_flags = required_flags;
 
   rule_set->rules = malloc (sizeof(OrcRule) * opcode_set->n_opcodes);
@@ -156,7 +176,7 @@ orc_target_get_rule (OrcTarget *target, OrcStaticOpcode *opcode,
     if (opcode_sets[k].opcodes + j != opcode) continue;
 
     for(i=0;i<target->n_rule_sets;i++){
-      if (target->rule_sets[i].opcode_set != opcode_sets + k) continue;
+      if (target->rule_sets[i].opcode_major != opcode_sets[k].opcode_major) continue;
       if (target->rule_sets[i].required_target_flags & (~target_flags)) continue;
 
       rule = target->rule_sets[i].rules + j;
@@ -206,6 +226,12 @@ orc_opcode_set_get (const char *name)
   return NULL;
 }
 
+OrcOpcodeSet *
+orc_opcode_set_get_nth (int opcode_major)
+{
+  return opcode_sets + opcode_major;
+}
+
 int
 orc_opcode_set_find_by_name (OrcOpcodeSet *opcode_set, const char *name)
 {
@@ -517,6 +543,20 @@ select1wb (OrcOpcodeExecutor *ex, void *user)
 #endif
 }
 
+static void
+splitlw (OrcOpcodeExecutor *ex, void *user)
+{
+  ex->dest_values[0] = (ex->src_values[0] >> 16) & 0xffff;
+  ex->dest_values[1] = ex->src_values[0] & 0xffff;
+}
+
+static void
+splitwb (OrcOpcodeExecutor *ex, void *user)
+{
+  ex->dest_values[0] = (ex->src_values[0] >> 8) & 0xff;
+  ex->dest_values[1] = ex->src_values[0] & 0xff;
+}
+
 static void
 mergewl (OrcOpcodeExecutor *ex, void *user)
 {
@@ -548,6 +588,88 @@ accsadubl (OrcOpcodeExecutor *ex, void *user)
       (int)((uint8_t)ex->src_values[1]));
 }
 
+/* float ops */
+
+static float
+ORC_FLOAT_READ(void *addr)
+{
+  union {
+    float f;
+    unsigned int i;
+  } x;
+  x.i = *(unsigned int *)(addr);
+  return x.f;
+}
+
+static void
+ORC_FLOAT_WRITE(void *addr, float value)
+{
+  union {
+    float f;
+    unsigned int i;
+  } x;
+  x.f = value;
+  *(unsigned int *)(addr) = x.i;
+}
+
+#if 0
+/* Oh noes!  Aliasing rules! */
+#define ORC_FLOAT_READ(addr) (*(float *)(addr))
+#define ORC_FLOAT_WRITE(addr,value) do{ (*(float *)(addr)) = (value); }while(0
+#endif
+
+#define UNARY_F(name,code) \
+static void \
+name (OrcOpcodeExecutor *ex, void *user) \
+{ \
+  float a = ORC_FLOAT_READ(&ex->src_values[0]); \
+  ORC_FLOAT_WRITE(&ex->dest_values[0], code ); \
+}
+
+#define BINARY_F(name,code) \
+static void \
+name (OrcOpcodeExecutor *ex, void *user) \
+{ \
+  void *pa = &ex->src_values[0]; \
+  void *pb = &ex->src_values[1]; \
+  float a = ORC_FLOAT_READ(pa); \
+  float b = ORC_FLOAT_READ(pb); \
+  ORC_FLOAT_WRITE(&ex->dest_values[0], code ); \
+}
+
+#define BINARY_FL(name,code) \
+static void \
+name (OrcOpcodeExecutor *ex, void *user) \
+{ \
+  float a = ORC_FLOAT_READ(&ex->src_values[0]); \
+  float b = ORC_FLOAT_READ(&ex->src_values[1]); \
+  ex->dest_values[0] = code ; \
+}
+
+BINARY_F(addf, a + b)
+BINARY_F(subf, a - b)
+BINARY_F(mulf, a * b)
+BINARY_F(divf, a / b)
+UNARY_F(orc_sqrtf, sqrt(a) )
+BINARY_F(maxf, (a>b) ? a : b)
+BINARY_F(minf, (a<b) ? a : b)
+
+BINARY_FL(cmpeqf, (a == b) ? (~0) : 0)
+BINARY_FL(cmpltf, (a < b) ? (~0) : 0)
+BINARY_FL(cmplef, (a <= b) ? (~0) : 0)
+
+static void
+convfl (OrcOpcodeExecutor *ex, void *user)
+{
+  ex->dest_values[0] = floor(0.5 + ORC_FLOAT_READ(&ex->src_values[0]));
+}
+
+static void
+convlf (OrcOpcodeExecutor *ex, void *user)
+{
+  ORC_FLOAT_WRITE(&ex->dest_values[0], ex->src_values[0]);
+}
+
 
 static OrcStaticOpcode opcodes[] = {
 
@@ -687,6 +809,22 @@ static OrcStaticOpcode opcodes[] = {
   { "select1lw", select1lw, NULL, 0, { 2 }, { 4 } },
   { "mergewl", mergewl, NULL, 0, { 4 }, { 2, 2 } },
   { "mergebw", mergebw, NULL, 0, { 2 }, { 1, 1 } },
+  { "splitlw", splitlw, NULL, 0, { 2, 2 }, { 4 } },
+  { "splitwb", splitwb, NULL, 0, { 1, 1 }, { 2 } },
+
+  /* float ops */
+  { "addf", addf, NULL, ORC_STATIC_OPCODE_FLOAT, { 4 }, { 4, 4 } },
+  { "subf", subf, NULL, ORC_STATIC_OPCODE_FLOAT, { 4 }, { 4, 4 } },
+  { "mulf", mulf, NULL, ORC_STATIC_OPCODE_FLOAT, { 4 }, { 4, 4 } },
+  { "divf", divf, NULL, ORC_STATIC_OPCODE_FLOAT, { 4 }, { 4, 4 } },
+  { "sqrtf", orc_sqrtf, NULL, ORC_STATIC_OPCODE_FLOAT, { 4 }, { 4 } },
+  { "maxf", maxf, NULL, ORC_STATIC_OPCODE_FLOAT, { 4 }, { 4, 4 } },
+  { "minf", minf, NULL, ORC_STATIC_OPCODE_FLOAT, { 4 }, { 4, 4 } },
+  { "cmpeqf", cmpeqf, NULL, ORC_STATIC_OPCODE_FLOAT_SRC, { 4 }, { 4, 4 } },
+  { "cmpltf", cmpltf, NULL, ORC_STATIC_OPCODE_FLOAT_SRC, { 4 }, { 4, 4 } },
+  { "cmplef", cmplef, NULL, ORC_STATIC_OPCODE_FLOAT_SRC, { 4 }, { 4, 4 } },
+  { "convfl", convfl, NULL, ORC_STATIC_OPCODE_FLOAT_SRC, { 4 }, { 4 } },
+  { "convlf", convlf, NULL, ORC_STATIC_OPCODE_FLOAT_DEST, { 4 }, { 4 } },
 
   { "" }
 };
