@@ -5,7 +5,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <unistd.h>
 #include <sys/types.h>
 
 #include <orc/orcprogram.h>
@@ -16,8 +15,9 @@
 
 #define SIZE 65536
 
+#define ORC_SSE_ALIGNED_DEST_CUTOFF 64
 
-void orc_sse_emit_loop (OrcCompiler *compiler);
+void orc_sse_emit_loop (OrcCompiler *compiler, int offset, int update);
 
 void orc_compiler_sse_init (OrcCompiler *compiler);
 unsigned int orc_compiler_sse_get_default_flags (void);
@@ -27,6 +27,7 @@ void orc_compiler_sse_register_rules (OrcTarget *target);
 
 void orc_compiler_rewrite_vars (OrcCompiler *compiler);
 void orc_compiler_dump (OrcCompiler *compiler);
+void sse_load_constant (OrcCompiler *compiler, int reg, int size, int value);
 
 static OrcTarget sse_target = {
   "sse",
@@ -38,7 +39,12 @@ static OrcTarget sse_target = {
   ORC_VEC_REG_BASE,
   orc_compiler_sse_get_default_flags,
   orc_compiler_sse_init,
-  orc_compiler_sse_assemble
+  orc_compiler_sse_assemble,
+  { { 0 } },
+  0,
+  NULL,
+  sse_load_constant
+
 };
 
 
@@ -198,7 +204,8 @@ sse_save_accumulators (OrcCompiler *compiler)
 
         if (compiler->vars[i].size == 2) {
           orc_x86_emit_mov_sse_reg (compiler, src, compiler->gp_tmpreg);
-          orc_x86_emit_mov_reg_memoffset (compiler, 2, compiler->gp_tmpreg,
+          orc_x86_emit_and_imm_reg (compiler, 4, 0xffff, compiler->gp_tmpreg);
+          orc_x86_emit_mov_reg_memoffset (compiler, 4, compiler->gp_tmpreg,
               (int)ORC_STRUCT_OFFSET(OrcExecutor, accumulators[i-ORC_VAR_A1]),
               compiler->exec_reg);
         } else {
@@ -216,7 +223,22 @@ sse_save_accumulators (OrcCompiler *compiler)
 }
 
 void
-sse_load_constants (OrcCompiler *compiler)
+sse_load_constant (OrcCompiler *compiler, int reg, int size, int value)
+{
+  if (size == 1) {
+    orc_sse_emit_loadib (compiler, reg, value);
+  } else if (size == 2) {
+    orc_sse_emit_loadiw (compiler, reg, value);
+  } else if (size == 4) {
+    orc_sse_emit_loadil (compiler, reg, value);
+  } else {
+    ORC_COMPILER_ERROR(compiler, "unimplemented");
+  }
+
+}
+
+void
+sse_load_constants_outer (OrcCompiler *compiler)
 {
   int i;
   for(i=0;i<ORC_N_VARIABLES;i++){
@@ -251,13 +273,6 @@ sse_load_constants (OrcCompiler *compiler)
         break;
       case ORC_VAR_TYPE_SRC:
       case ORC_VAR_TYPE_DEST:
-        if (compiler->vars[i].ptr_register) {
-          orc_x86_emit_mov_memoffset_reg (compiler, compiler->is_64bit ? 8 : 4,
-              (int)ORC_STRUCT_OFFSET(OrcExecutor, arrays[i]), compiler->exec_reg,
-              compiler->vars[i].ptr_register);
-        } else {
-          ORC_COMPILER_ERROR(compiler,"unimplemented");
-        }
         break;
       case ORC_VAR_TYPE_ACCUMULATOR:
         orc_sse_emit_660f (compiler, "pxor", 0xef,
@@ -273,7 +288,71 @@ sse_load_constants (OrcCompiler *compiler)
 }
 
 void
-orc_sse_emit_load_src (OrcCompiler *compiler, OrcVariable *var)
+sse_load_constants_inner (OrcCompiler *compiler)
+{
+  int i;
+  for(i=0;i<ORC_N_VARIABLES;i++){
+    if (compiler->vars[i].name == NULL) continue;
+    switch (compiler->vars[i].vartype) {
+      case ORC_VAR_TYPE_CONST:
+        break;
+      case ORC_VAR_TYPE_PARAM:
+        break;
+      case ORC_VAR_TYPE_SRC:
+      case ORC_VAR_TYPE_DEST:
+        if (compiler->vars[i].ptr_register) {
+          orc_x86_emit_mov_memoffset_reg (compiler, compiler->is_64bit ? 8 : 4,
+              (int)ORC_STRUCT_OFFSET(OrcExecutor, arrays[i]), compiler->exec_reg,
+              compiler->vars[i].ptr_register);
+        } else {
+          ORC_COMPILER_ERROR(compiler,"unimplemented");
+        }
+        break;
+      case ORC_VAR_TYPE_ACCUMULATOR:
+        break;
+      case ORC_VAR_TYPE_TEMP:
+        break;
+      default:
+        ORC_COMPILER_ERROR(compiler,"bad vartype");
+        break;
+    }
+  }
+}
+
+void
+sse_add_strides (OrcCompiler *compiler)
+{
+  int i;
+
+  for(i=0;i<ORC_N_VARIABLES;i++){
+    if (compiler->vars[i].name == NULL) continue;
+    switch (compiler->vars[i].vartype) {
+      case ORC_VAR_TYPE_CONST:
+        break;
+      case ORC_VAR_TYPE_PARAM:
+        break;
+      case ORC_VAR_TYPE_SRC:
+      case ORC_VAR_TYPE_DEST:
+        orc_x86_emit_mov_memoffset_reg (compiler, 4,
+            (int)ORC_STRUCT_OFFSET(OrcExecutor, params[i]), compiler->exec_reg,
+            X86_ECX);
+        orc_x86_emit_add_reg_memoffset (compiler, compiler->is_64bit ? 8 : 4,
+            X86_ECX,
+            (int)ORC_STRUCT_OFFSET(OrcExecutor, arrays[i]), compiler->exec_reg);
+        break;
+      case ORC_VAR_TYPE_ACCUMULATOR:
+        break;
+      case ORC_VAR_TYPE_TEMP:
+        break;
+      default:
+        ORC_COMPILER_ERROR(compiler,"bad vartype");
+        break;
+    }
+  }
+}
+
+void
+orc_sse_emit_load_src (OrcCompiler *compiler, OrcVariable *var, int offset)
 {
   int ptr_reg;
   if (var->ptr_register == 0) {
@@ -288,23 +367,23 @@ orc_sse_emit_load_src (OrcCompiler *compiler, OrcVariable *var)
   }
   switch (var->size << compiler->loop_shift) {
     case 1:
-      orc_x86_emit_mov_memoffset_reg (compiler, 1, 0, ptr_reg, compiler->gp_tmpreg);
+      orc_x86_emit_mov_memoffset_reg (compiler, 1, offset, ptr_reg, compiler->gp_tmpreg);
       orc_x86_emit_mov_reg_sse (compiler, compiler->gp_tmpreg, var->alloc);
       break;
     case 2:
-      orc_x86_emit_mov_memoffset_reg (compiler, 2, 0, ptr_reg, compiler->gp_tmpreg);
+      orc_x86_emit_mov_memoffset_reg (compiler, 2, offset, ptr_reg, compiler->gp_tmpreg);
       orc_x86_emit_mov_reg_sse (compiler, compiler->gp_tmpreg, var->alloc);
       break;
     case 4:
-      orc_x86_emit_mov_memoffset_sse (compiler, 4, 0, ptr_reg, var->alloc,
+      orc_x86_emit_mov_memoffset_sse (compiler, 4, offset, ptr_reg, var->alloc,
           var->is_aligned);
       break;
     case 8:
-      orc_x86_emit_mov_memoffset_sse (compiler, 8, 0, ptr_reg, var->alloc,
+      orc_x86_emit_mov_memoffset_sse (compiler, 8, offset, ptr_reg, var->alloc,
           var->is_aligned);
       break;
     case 16:
-      orc_x86_emit_mov_memoffset_sse (compiler, 16, 0, ptr_reg, var->alloc,
+      orc_x86_emit_mov_memoffset_sse (compiler, 16, offset, ptr_reg, var->alloc,
           var->is_aligned);
       break;
     default:
@@ -315,7 +394,7 @@ orc_sse_emit_load_src (OrcCompiler *compiler, OrcVariable *var)
 }
 
 void
-orc_sse_emit_store_dest (OrcCompiler *compiler, OrcVariable *var)
+orc_sse_emit_store_dest (OrcCompiler *compiler, OrcVariable *var, int offset)
 {
   int ptr_reg;
   if (var->ptr_register == 0) {
@@ -332,7 +411,7 @@ orc_sse_emit_store_dest (OrcCompiler *compiler, OrcVariable *var)
         ORC_COMPILER_ERROR(compiler,"unimplemented");
       }
       orc_x86_emit_mov_sse_reg (compiler, var->alloc, compiler->gp_tmpreg);
-      orc_x86_emit_mov_reg_memoffset (compiler, 1, compiler->gp_tmpreg, 0, ptr_reg);
+      orc_x86_emit_mov_reg_memoffset (compiler, 1, compiler->gp_tmpreg, offset, ptr_reg);
       break;
     case 2:
       /* FIXME we might be using ecx twice here */
@@ -340,18 +419,18 @@ orc_sse_emit_store_dest (OrcCompiler *compiler, OrcVariable *var)
         ORC_COMPILER_ERROR(compiler,"unimplemented");
       }
       orc_x86_emit_mov_sse_reg (compiler, var->alloc, compiler->gp_tmpreg);
-      orc_x86_emit_mov_reg_memoffset (compiler, 2, compiler->gp_tmpreg, 0, ptr_reg);
+      orc_x86_emit_mov_reg_memoffset (compiler, 2, compiler->gp_tmpreg, offset, ptr_reg);
       break;
     case 4:
-      orc_x86_emit_mov_sse_memoffset (compiler, 4, var->alloc, 0, ptr_reg,
+      orc_x86_emit_mov_sse_memoffset (compiler, 4, var->alloc, offset, ptr_reg,
           var->is_aligned, var->is_uncached);
       break;
     case 8:
-      orc_x86_emit_mov_sse_memoffset (compiler, 8, var->alloc, 0, ptr_reg,
+      orc_x86_emit_mov_sse_memoffset (compiler, 8, var->alloc, offset, ptr_reg,
           var->is_aligned, var->is_uncached);
       break;
     case 16:
-      orc_x86_emit_mov_sse_memoffset (compiler, 16, var->alloc, 0, ptr_reg,
+      orc_x86_emit_mov_sse_memoffset (compiler, 16, var->alloc, offset, ptr_reg,
           var->is_aligned, var->is_uncached);
       break;
     default:
@@ -389,8 +468,9 @@ get_shift (int size)
   return -1;
 }
 
+
 void
-orc_compiler_sse_assemble (OrcCompiler *compiler)
+orc_emit_split_n_regions (OrcCompiler *compiler)
 {
   int align_var;
   int align_shift;
@@ -400,124 +480,204 @@ orc_compiler_sse_assemble (OrcCompiler *compiler)
   var_size_shift = get_shift (compiler->vars[align_var].size);
   align_shift = var_size_shift + compiler->loop_shift;
 
+  /* determine how many iterations until align array is aligned (n1) */
+  orc_x86_emit_mov_imm_reg (compiler, 4, 16, X86_EAX);
+  orc_x86_emit_sub_memoffset_reg (compiler, 4,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor, arrays[align_var]),
+      compiler->exec_reg, X86_EAX);
+  orc_x86_emit_and_imm_reg (compiler, 4, (1<<align_shift) - 1, X86_EAX);
+  orc_x86_emit_sar_imm_reg (compiler, 4, var_size_shift, X86_EAX);
+
+  /* check if n1 is greater than n. */
+  orc_x86_emit_cmp_reg_memoffset (compiler, 4, X86_EAX,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg);
+
+  orc_x86_emit_jle (compiler, 6);
+
+  /* If so, we have a standard 3-region split. */
+  orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor,counter1), compiler->exec_reg);
+    
+  /* Calculate n2 */
+  orc_x86_emit_mov_memoffset_reg (compiler, 4,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg,
+      compiler->gp_tmpreg);
+  orc_x86_emit_sub_reg_reg (compiler, 4, X86_EAX, compiler->gp_tmpreg);
+
+  orc_x86_emit_mov_reg_reg (compiler, 4, compiler->gp_tmpreg, X86_EAX);
+
+  orc_x86_emit_sar_imm_reg (compiler, 4, compiler->loop_shift,
+      compiler->gp_tmpreg);
+  orc_x86_emit_mov_reg_memoffset (compiler, 4, compiler->gp_tmpreg,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor,counter2), compiler->exec_reg);
+
+  /* Calculate n3 */
+  orc_x86_emit_and_imm_reg (compiler, 4, (1<<compiler->loop_shift)-1, X86_EAX);
+  orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor,counter3), compiler->exec_reg);
+
+  orc_x86_emit_jmp (compiler, 7);
+
+  /* else, iterations are all unaligned: n1=n, n2=0, n3=0 */
+  orc_x86_emit_label (compiler, 6);
+
+  orc_x86_emit_mov_memoffset_reg (compiler, 4,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg, X86_EAX);
+  orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor,counter1), compiler->exec_reg);
+  orc_x86_emit_mov_imm_reg (compiler, 4, 0, X86_EAX);
+  orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor,counter2), compiler->exec_reg);
+  orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
+      (int)ORC_STRUCT_OFFSET(OrcExecutor,counter3), compiler->exec_reg);
+
+  orc_x86_emit_label (compiler, 7);
+}
+
+void
+orc_compiler_sse_assemble (OrcCompiler *compiler)
+{
+  int align_var;
+
+  align_var = get_align_var (compiler);
+
   compiler->vars[align_var].is_aligned = FALSE;
 
   orc_x86_emit_prologue (compiler);
 
-  if (compiler->loop_shift > 0) {
-    orc_x86_emit_mov_imm_reg (compiler, 4, 16, X86_EAX);
-    orc_x86_emit_sub_memoffset_reg (compiler, 4,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor, arrays[align_var]),
-        compiler->exec_reg, X86_EAX);
-    orc_x86_emit_and_imm_reg (compiler, 4, (1<<align_shift) - 1, X86_EAX);
-    orc_x86_emit_sar_imm_reg (compiler, 4, var_size_shift, X86_EAX);
+  sse_load_constants_outer (compiler);
 
-    orc_x86_emit_cmp_reg_memoffset (compiler, 4, X86_EAX,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg);
+  if (compiler->program->is_2d) {
+    if (compiler->program->constant_m > 0) {
+      orc_x86_emit_mov_imm_reg (compiler, 4, compiler->program->constant_m,
+          X86_EAX);
+      orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
+          (int)ORC_STRUCT_OFFSET(OrcExecutor, params[ORC_VAR_A2]),
+          compiler->exec_reg);
+    } else {
+      orc_x86_emit_mov_memoffset_reg (compiler, 4,
+          (int)ORC_STRUCT_OFFSET(OrcExecutor, params[ORC_VAR_A1]),
+          compiler->exec_reg, X86_EAX);
+      orc_x86_emit_test_reg_reg (compiler, 4, X86_EAX, X86_EAX);
+      orc_x86_emit_jle (compiler, 17);
+      orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
+          (int)ORC_STRUCT_OFFSET(OrcExecutor, params[ORC_VAR_A2]),
+          compiler->exec_reg);
+    }
 
-    orc_x86_emit_jle (compiler, 6);
+    orc_x86_emit_label (compiler, 16);
+  }
 
-    orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor,counter1), compiler->exec_reg);
-    
-    orc_x86_emit_mov_memoffset_reg (compiler, 4,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg,
-        compiler->gp_tmpreg);
-    orc_x86_emit_sub_reg_reg (compiler, 4, X86_EAX, compiler->gp_tmpreg);
-
-    orc_x86_emit_mov_reg_reg (compiler, 4, compiler->gp_tmpreg, X86_EAX);
-
-    orc_x86_emit_sar_imm_reg (compiler, 4, compiler->loop_shift,
-        compiler->gp_tmpreg);
-    orc_x86_emit_mov_reg_memoffset (compiler, 4, compiler->gp_tmpreg,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor,counter2), compiler->exec_reg);
-
-    orc_x86_emit_and_imm_reg (compiler, 4, (1<<compiler->loop_shift)-1, X86_EAX);
-    orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor,counter3), compiler->exec_reg);
-
-    orc_x86_emit_jmp (compiler, 7);
-    orc_x86_emit_label (compiler, 6);
-
-    orc_x86_emit_mov_memoffset_reg (compiler, 4,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg, X86_EAX);
-    orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor,counter1), compiler->exec_reg);
-    orc_x86_emit_mov_imm_reg (compiler, 4, 0, X86_EAX);
-    orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor,counter2), compiler->exec_reg);
-    orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
-        (int)ORC_STRUCT_OFFSET(OrcExecutor,counter3), compiler->exec_reg);
-
-    orc_x86_emit_label (compiler, 7);
+  if (compiler->program->constant_n > 0 &&
+      compiler->program->constant_n <= ORC_SSE_ALIGNED_DEST_CUTOFF) {
+    /* don't need to load n */
+  } else if (compiler->loop_shift > 0) {
+    /* split n into three regions, with center region being aligned */
+    orc_emit_split_n_regions (compiler);
   } else {
+    /* loop shift is 0, no need to split */
     orc_x86_emit_mov_memoffset_reg (compiler, 4,
         (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg, compiler->gp_tmpreg);
     orc_x86_emit_mov_reg_memoffset (compiler, 4, compiler->gp_tmpreg,
         (int)ORC_STRUCT_OFFSET(OrcExecutor,counter2), compiler->exec_reg);
   }
 
-  sse_load_constants (compiler);
+  sse_load_constants_inner (compiler);
 
-  if (compiler->loop_shift > 0) {
+  if (compiler->program->constant_n > 0 &&
+      compiler->program->constant_n <= ORC_SSE_ALIGNED_DEST_CUTOFF) {
+    int n_left = compiler->program->constant_n;
     int save_loop_shift;
-    int l;
+    int loop_shift;
+    int offset = 0;
 
     save_loop_shift = compiler->loop_shift;
-    compiler->vars[align_var].is_aligned = FALSE;
-
-    for (l=0;l<save_loop_shift;l++){
-      compiler->loop_shift = l;
+    while (n_left >= (1<<compiler->loop_shift)) {
       ORC_ASM_CODE(compiler, "# LOOP SHIFT %d\n", compiler->loop_shift);
+      orc_sse_emit_loop (compiler, offset, FALSE);
 
-      orc_x86_emit_test_imm_memoffset (compiler, 4, 1<<compiler->loop_shift,
-          (int)ORC_STRUCT_OFFSET(OrcExecutor,counter1), compiler->exec_reg);
-      orc_x86_emit_je (compiler, 12 + compiler->loop_shift);
-      orc_sse_emit_loop (compiler);
-      orc_x86_emit_label (compiler, 12 + compiler->loop_shift);
+      n_left -= 1<<compiler->loop_shift;
+      offset += 1<<compiler->loop_shift;
+    }
+    for(loop_shift = compiler->loop_shift-1; loop_shift>=0; loop_shift--) {
+      if (n_left >= (1<<loop_shift)) {
+        compiler->loop_shift = loop_shift;
+        ORC_ASM_CODE(compiler, "# LOOP SHIFT %d\n", loop_shift);
+        orc_sse_emit_loop (compiler, offset, FALSE);
+        n_left -= 1<<loop_shift;
+        offset += 1<<loop_shift;
+      }
+    }
+    compiler->loop_shift = save_loop_shift;
+  } else {
+    if (compiler->loop_shift > 0) {
+      int save_loop_shift;
+      int l;
+
+      save_loop_shift = compiler->loop_shift;
+      compiler->vars[align_var].is_aligned = FALSE;
+
+      for (l=0;l<save_loop_shift;l++){
+        compiler->loop_shift = l;
+        ORC_ASM_CODE(compiler, "# LOOP SHIFT %d\n", compiler->loop_shift);
+
+        orc_x86_emit_test_imm_memoffset (compiler, 4, 1<<compiler->loop_shift,
+            (int)ORC_STRUCT_OFFSET(OrcExecutor,counter1), compiler->exec_reg);
+        orc_x86_emit_je (compiler, 12 + compiler->loop_shift);
+        orc_sse_emit_loop (compiler, 0, TRUE);
+        orc_x86_emit_label (compiler, 12 + compiler->loop_shift);
+      }
+
+      compiler->loop_shift = save_loop_shift;
+      compiler->vars[align_var].is_aligned = TRUE;
     }
 
-    compiler->loop_shift = save_loop_shift;
+    orc_x86_emit_label (compiler, 1);
 
-    compiler->loop_shift = save_loop_shift;
-    compiler->vars[align_var].is_aligned = TRUE;
+    orc_x86_emit_cmp_imm_memoffset (compiler, 4, 0,
+        (int)ORC_STRUCT_OFFSET(OrcExecutor,counter2), compiler->exec_reg);
+    orc_x86_emit_je (compiler, 3);
+
+    ORC_ASM_CODE(compiler, "# LOOP SHIFT %d\n", compiler->loop_shift);
+    orc_x86_emit_align (compiler);
+    orc_x86_emit_label (compiler, 2);
+    orc_sse_emit_loop (compiler, 0, TRUE);
+    orc_x86_emit_dec_memoffset (compiler, 4,
+        (int)ORC_STRUCT_OFFSET(OrcExecutor,counter2),
+        compiler->exec_reg);
+    orc_x86_emit_jne (compiler, 2);
+    orc_x86_emit_label (compiler, 3);
+
+    if (compiler->loop_shift > 0) {
+      int save_loop_shift;
+      int l;
+
+      save_loop_shift = compiler->loop_shift;
+      compiler->vars[align_var].is_aligned = FALSE;
+
+      for(l=save_loop_shift - 1; l >= 0; l--) {
+        compiler->loop_shift = l;
+        ORC_ASM_CODE(compiler, "# LOOP SHIFT %d\n", compiler->loop_shift);
+
+        orc_x86_emit_test_imm_memoffset (compiler, 4, 1<<compiler->loop_shift,
+            (int)ORC_STRUCT_OFFSET(OrcExecutor,counter3), compiler->exec_reg);
+        orc_x86_emit_je (compiler, 8 + compiler->loop_shift);
+        orc_sse_emit_loop (compiler, 0, TRUE);
+        orc_x86_emit_label (compiler, 8 + compiler->loop_shift);
+      }
+
+      compiler->loop_shift = save_loop_shift;
+    }
   }
 
-  orc_x86_emit_label (compiler, 1);
+  if (compiler->program->is_2d) {
+    sse_add_strides (compiler);
 
-  orc_x86_emit_cmp_imm_memoffset (compiler, 4, 0,
-      (int)ORC_STRUCT_OFFSET(OrcExecutor,counter2), compiler->exec_reg);
-  orc_x86_emit_je (compiler, 3);
-
-  ORC_ASM_CODE(compiler, "# LOOP SHIFT %d\n", compiler->loop_shift);
-  orc_x86_emit_align (compiler);
-  orc_x86_emit_label (compiler, 2);
-  orc_sse_emit_loop (compiler);
-  orc_x86_emit_dec_memoffset (compiler, 4,
-      (int)ORC_STRUCT_OFFSET(OrcExecutor,counter2),
-      compiler->exec_reg);
-  orc_x86_emit_jne (compiler, 2);
-  orc_x86_emit_label (compiler, 3);
-
-  if (compiler->loop_shift > 0) {
-    int save_loop_shift;
-    int l;
-
-    save_loop_shift = compiler->loop_shift;
-    compiler->vars[align_var].is_aligned = FALSE;
-
-    for(l=save_loop_shift - 1; l >= 0; l--) {
-      compiler->loop_shift = l;
-      ORC_ASM_CODE(compiler, "# LOOP SHIFT %d\n", compiler->loop_shift);
-
-      orc_x86_emit_test_imm_memoffset (compiler, 4, 1<<compiler->loop_shift,
-          (int)ORC_STRUCT_OFFSET(OrcExecutor,counter3), compiler->exec_reg);
-      orc_x86_emit_je (compiler, 8 + compiler->loop_shift);
-      orc_sse_emit_loop (compiler);
-      orc_x86_emit_label (compiler, 8 + compiler->loop_shift);
-    }
-
-    compiler->loop_shift = save_loop_shift;
+    orc_x86_emit_add_imm_memoffset (compiler, 4, -1,
+        (int)ORC_STRUCT_OFFSET(OrcExecutor,params[ORC_VAR_A2]),
+        compiler->exec_reg);
+    orc_x86_emit_jne (compiler, 16);
+    orc_x86_emit_label (compiler, 17);
   }
 
   sse_save_accumulators (compiler);
@@ -528,13 +688,12 @@ orc_compiler_sse_assemble (OrcCompiler *compiler)
 }
 
 void
-orc_sse_emit_loop (OrcCompiler *compiler)
+orc_sse_emit_loop (OrcCompiler *compiler, int offset, int update)
 {
   int j;
   int k;
   OrcInstruction *insn;
   OrcStaticOpcode *opcode;
-  //OrcVariable *args[10];
   OrcRule *rule;
 
   for(j=0;j<compiler->n_insns;j++){
@@ -563,7 +722,7 @@ orc_sse_emit_loop (OrcCompiler *compiler)
       switch (var->vartype) {
         case ORC_VAR_TYPE_SRC:
         case ORC_VAR_TYPE_DEST:
-          orc_sse_emit_load_src (compiler, var);
+          orc_sse_emit_load_src (compiler, var, offset*var->size);
           break;
         case ORC_VAR_TYPE_CONST:
           break;
@@ -597,7 +756,7 @@ orc_sse_emit_loop (OrcCompiler *compiler)
 
       switch (var->vartype) {
         case ORC_VAR_TYPE_DEST:
-          orc_sse_emit_store_dest (compiler, var);
+          orc_sse_emit_store_dest (compiler, var, offset*var->size);
           break;
         case ORC_VAR_TYPE_TEMP:
           break;
@@ -607,19 +766,21 @@ orc_sse_emit_loop (OrcCompiler *compiler)
     }
   }
 
-  for(k=0;k<ORC_N_VARIABLES;k++){
-    if (compiler->vars[k].name == NULL) continue;
-    if (compiler->vars[k].vartype == ORC_VAR_TYPE_SRC ||
-        compiler->vars[k].vartype == ORC_VAR_TYPE_DEST) {
-      if (compiler->vars[k].ptr_register) {
-        orc_x86_emit_add_imm_reg (compiler, compiler->is_64bit ? 8 : 4,
-            compiler->vars[k].size << compiler->loop_shift,
-            compiler->vars[k].ptr_register);
-      } else {
-        orc_x86_emit_add_imm_memoffset (compiler, compiler->is_64bit ? 8 : 4,
-            compiler->vars[k].size << compiler->loop_shift,
-            (int)ORC_STRUCT_OFFSET(OrcExecutor, arrays[k]),
-            compiler->exec_reg);
+  if (update) {
+    for(k=0;k<ORC_N_VARIABLES;k++){
+      if (compiler->vars[k].name == NULL) continue;
+      if (compiler->vars[k].vartype == ORC_VAR_TYPE_SRC ||
+          compiler->vars[k].vartype == ORC_VAR_TYPE_DEST) {
+        if (compiler->vars[k].ptr_register) {
+          orc_x86_emit_add_imm_reg (compiler, compiler->is_64bit ? 8 : 4,
+              compiler->vars[k].size << compiler->loop_shift,
+              compiler->vars[k].ptr_register);
+        } else {
+          orc_x86_emit_add_imm_memoffset (compiler, compiler->is_64bit ? 8 : 4,
+              compiler->vars[k].size << compiler->loop_shift,
+              (int)ORC_STRUCT_OFFSET(OrcExecutor, arrays[k]),
+              compiler->exec_reg);
+        }
       }
     }
   }
