@@ -147,6 +147,9 @@ orc_program_free (OrcProgram *program)
   for(i=0;i<ORC_N_VARIABLES;i++){
     if (program->vars[i].name) free (program->vars[i].name);
   }
+  if (program->asm_code) {
+    free (program->asm_code);
+  }
   if (program->name) {
     free (program->name);
   }
@@ -169,6 +172,30 @@ orc_program_set_name (OrcProgram *program, const char *name)
   program->name = strdup (name);
 }
 
+/**
+ * orc_program_set_2d:
+ * @program: a pointer to an OrcProgram structure
+ *
+ * Sets a flag on the program indicating that arrays are two
+ * dimensional.  This causes the compiler to generate code for
+ * an OrcExec2D executor.
+ */
+void
+orc_program_set_2d (OrcProgram *program)
+{
+  program->is_2d = TRUE;
+}
+
+void orc_program_set_constant_n (OrcProgram *program, int n)
+{
+  program->constant_n = n;
+}
+
+void orc_program_set_constant_m (OrcProgram *program, int m)
+{
+  program->constant_m = m;
+}
+
 /**
  * orc_program_set_backup_function:
  * @program: a pointer to an OrcProgram structure
@@ -373,6 +400,13 @@ orc_program_set_type_name (OrcProgram *program, int var, const char *type_name)
   program->vars[var].type_name = strdup(type_name);
 }
 
+void
+orc_program_set_sampling_type (OrcProgram *program, int var,
+    int sampling_type)
+{
+  program->vars[var].sampling_type = sampling_type;
+}
+
 /**
  * orc_program_append_ds:
  * @program: a pointer to an OrcProgram structure
@@ -513,6 +547,25 @@ orc_program_append_ds_str (OrcProgram *program, const char *name,
   program->n_insns++;
 }
 
+void
+orc_program_append_dds_str (OrcProgram *program, const char *name,
+    const char *arg1, const char *arg2, const char *arg3)
+{
+  OrcInstruction *insn;
+
+  insn = program->insns + program->n_insns;
+
+  insn->opcode = orc_opcode_find_by_name (name);
+  if (!insn->opcode) {
+    ORC_ERROR ("unknown opcode: %s", name);
+  }
+  insn->dest_args[0] = orc_program_find_var_by_name (program, arg1);
+  insn->dest_args[1] = orc_program_find_var_by_name (program, arg2);
+  insn->src_args[0] = orc_program_find_var_by_name (program, arg3);
+  
+  program->n_insns++;
+}
+
 /**
  * orc_program_get_asm_code:
  * @program: a pointer to an OrcProgram structure
@@ -545,7 +598,8 @@ orc_program_get_max_var_size (OrcProgram *program)
 
   max = 0;
   for(i=0;i<ORC_N_VARIABLES;i++){
-    if (program->vars[i].size) {
+    if (program->vars[i].size &&
+        program->vars[i].vartype != ORC_VAR_TYPE_ACCUMULATOR) {
       max = MAX(max, program->vars[i].size);
     }
   }
@@ -580,3 +634,29 @@ orc_program_get_max_array_size (OrcProgram *program)
   return max;
 }
 
+/**
+ * orc_program_get_max_accumulator_size:
+ * @program: a pointer to an OrcProgram structure
+ *
+ * Returns the size of the largest array used in the program.
+ * 
+ * Returns: the number of bytes
+ */
+int
+orc_program_get_max_accumulator_size (OrcProgram *program)
+{
+  int i;
+  int max;
+
+  max = 0;
+  for(i=0;i<ORC_N_VARIABLES;i++){
+    if (program->vars[i].size) {
+      if (program->vars[i].vartype == ORC_VAR_TYPE_ACCUMULATOR) {
+        max = MAX(max, program->vars[i].size);
+      }
+    }
+  }
+
+  return max;
+}
+
