@@ -5,7 +5,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <unistd.h>
 #include <sys/types.h>
 
 #include <orc/orcprogram.h>
@@ -20,7 +19,7 @@ void
 orc_sse_emit_loadil (OrcCompiler *p, int reg, int value)
 {
   if (value == 0) {
-    orc_sse_emit_660f (p, "pxor", 0xef, reg, reg);
+    orc_sse_emit_pxor(p, reg, reg);
   } else {
     orc_x86_emit_mov_imm_reg (p, 4, value, p->gp_tmpreg);
     orc_x86_emit_mov_reg_sse (p, p->gp_tmpreg, reg);
@@ -48,12 +47,11 @@ orc_sse_emit_loadiw (OrcCompiler *p, int reg, int value)
 void
 orc_sse_emit_loadpb (OrcCompiler *p, int reg, int param)
 {
-  orc_x86_emit_mov_memoffset_sse (p, 4, 
+  orc_x86_emit_mov_memoffset_sse (p, 4,
       (int)ORC_STRUCT_OFFSET(OrcExecutor, params[param]),
       p->exec_reg, reg, FALSE);
 
-  orc_sse_emit_660f (p, "punpcklbw", 0x60, reg, reg);
-
+  orc_sse_emit_punpcklbw (p, reg, reg);
   orc_sse_emit_pshuflw (p, 0, reg, reg);
   orc_sse_emit_pshufd (p, 0, reg, reg);
 }
@@ -61,7 +59,7 @@ orc_sse_emit_loadpb (OrcCompiler *p, int reg, int param)
 void
 orc_sse_emit_loadpw (OrcCompiler *p, int reg, int param)
 {
-  orc_x86_emit_mov_memoffset_sse (p, 4, 
+  orc_x86_emit_mov_memoffset_sse (p, 4,
       (int)ORC_STRUCT_OFFSET(OrcExecutor, params[param]),
       p->exec_reg, reg, FALSE);
 
@@ -72,7 +70,7 @@ orc_sse_emit_loadpw (OrcCompiler *p, int reg, int param)
 void
 orc_sse_emit_loadpl (OrcCompiler *p, int reg, int param)
 {
-  orc_x86_emit_mov_memoffset_sse (p, 4, 
+  orc_x86_emit_mov_memoffset_sse (p, 4,
       (int)ORC_STRUCT_OFFSET(OrcExecutor, params[param]),
       p->exec_reg, reg, FALSE);
 
@@ -82,7 +80,7 @@ orc_sse_emit_loadpl (OrcCompiler *p, int reg, int param)
 void
 orc_sse_emit_loadpq (OrcCompiler *p, int reg, int param)
 {
-  orc_x86_emit_mov_memoffset_sse (p, 8, 
+  orc_x86_emit_mov_memoffset_sse (p, 8,
       (int)ORC_STRUCT_OFFSET(OrcExecutor, params[param]),
       p->exec_reg, reg, FALSE);
 
@@ -92,7 +90,7 @@ orc_sse_emit_loadpq (OrcCompiler *p, int reg, int param)
 static void
 sse_rule_copyx (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
-  orc_sse_emit_660f (p, "movdqa", 0x6f,
+  orc_sse_emit_movdqa (p,
       p->vars[insn->src_args[0]].alloc,
       p->vars[insn->dest_args[0]].alloc);
 }
@@ -192,7 +190,7 @@ sse_rule_accw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
 
-  orc_sse_emit_660f (p, "paddw", 0xfd, src, dest);
+  orc_sse_emit_paddw (p, src, dest);
 }
 
 static void
@@ -202,9 +200,9 @@ sse_rule_accl (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = p->vars[insn->dest_args[0]].alloc;
 
   if (p->loop_shift == 0) {
-    orc_sse_emit_shiftimm (p, "pslldq", 0x73, 7, 12, src);
+    orc_sse_emit_pslldq (p, 12, src);
   }
-  orc_sse_emit_660f (p, "paddd", 0xfe, src, dest);
+  orc_sse_emit_paddd (p, src, dest);
 }
 
 static void
@@ -213,10 +211,11 @@ sse_rule_accsadubl (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src1 = p->vars[insn->src_args[0]].alloc;
   int src2 = p->vars[insn->src_args[1]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
+  int tmp = p->tmpreg;
 
-  orc_sse_emit_660f (p, "movdqa", 0x6f, src1, p->tmpreg);
-  orc_sse_emit_660f (p, "psadbw", 0xf6, src2, p->tmpreg);
-  orc_sse_emit_660f (p, "paddd", 0xfe, p->tmpreg, dest);
+  orc_sse_emit_movdqa (p, src1, tmp);
+  orc_sse_emit_psadbw (p, src2, tmp);
+  orc_sse_emit_paddd (p, tmp, dest);
 }
 
 static void
@@ -224,21 +223,20 @@ sse_rule_signX_ssse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
-  int imm_vals[] = { 0x01010101, 0x00010001, 0x00000001 };
   const char * names[] = { "psignb", "psignw", "psignd" };
   int codes[] = { 0x3808, 0x3809, 0x380a };
+  int type = ORC_PTR_TO_INT(user);
+  int tmpc;
 
   if (src == dest) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, p->tmpreg);
-    src = p->tmpreg;
+    tmpc = orc_compiler_get_constant (p, 1<<type, 1);
+    orc_sse_emit_660f (p, names[type], codes[type], src, tmpc);
+    orc_sse_emit_movdqa (p, tmpc, dest);
+  } else {
+    tmpc = orc_compiler_get_constant (p, 1<<type, 1);
+    orc_sse_emit_movdqa (p, tmpc, dest);
+    orc_sse_emit_660f (p, names[type], codes[type], src, dest);
   }
-
-  orc_x86_emit_mov_imm_reg (p, 4, imm_vals[ORC_PTR_TO_INT(user)], p->gp_tmpreg);
-
-  orc_x86_emit_mov_reg_sse (p, p->gp_tmpreg, dest);
-  orc_sse_emit_pshufd (p, 0, dest, dest);
-
-  orc_sse_emit_660f (p, names[ORC_PTR_TO_INT(user)], codes[ORC_PTR_TO_INT(user)], src, dest);
 }
 
 static void
@@ -246,20 +244,22 @@ sse_rule_signw_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
+  int tmp = p->tmpreg;
+  int gptmp = p->gp_tmpreg;
 
   if (src != dest) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_x86_emit_mov_imm_reg (p, 4, 0x00010001, p->gp_tmpreg);
-  orc_x86_emit_mov_reg_sse (p, p->gp_tmpreg, p->tmpreg);
-  orc_sse_emit_pshufd (p, 0, p->tmpreg, p->tmpreg);
-  orc_sse_emit_660f (p, "pminsw", 0xea, p->tmpreg, dest);
+  orc_x86_emit_mov_imm_reg (p, 4, 0x00010001, gptmp);
+  orc_x86_emit_mov_reg_sse (p, gptmp, tmp);
+  orc_sse_emit_pshufd (p, 0, tmp, tmp);
+  orc_sse_emit_pminsw (p, tmp, dest);
 
-  orc_x86_emit_mov_imm_reg (p, 4, 0xffffffff, p->gp_tmpreg);
-  orc_x86_emit_mov_reg_sse (p, p->gp_tmpreg, p->tmpreg);
-  orc_sse_emit_pshufd (p, 0, p->tmpreg, p->tmpreg);
-  orc_sse_emit_660f (p, "pmaxsw", 0xee, p->tmpreg, dest);
+  orc_x86_emit_mov_imm_reg (p, 4, 0xffffffff, gptmp);
+  orc_x86_emit_mov_reg_sse (p, gptmp, tmp);
+  orc_sse_emit_pshufd (p, 0, tmp, tmp);
+  orc_sse_emit_pmaxsw (p, tmp, dest);
 }
 
 static void
@@ -267,20 +267,18 @@ sse_rule_absw_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
-  int tmp;
+  int tmp = p->tmpreg;
 
   if (src == dest) {
-    tmp = p->tmpreg;
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, tmp);
+    orc_sse_emit_movdqa (p, src, tmp);
   } else {
-    tmp = p->tmpreg;
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, tmp);
-    orc_sse_emit_660f (p, "movdqa", 0x6f, tmp, dest);
+    orc_sse_emit_movdqa (p, src, tmp);
+    orc_sse_emit_movdqa (p, tmp, dest);
   }
 
-  orc_sse_emit_shiftimm (p, "psraw", 0x71, 4, 15, tmp);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
-  orc_sse_emit_660f (p, "psubw", 0xf9, tmp, dest);
+  orc_sse_emit_psraw (p, 15, tmp);
+  orc_sse_emit_pxor (p, tmp, dest);
+  orc_sse_emit_psubw (p, tmp, dest);
 
 }
 
@@ -300,7 +298,7 @@ sse_rule_shift (OrcCompiler *p, void *user, OrcInstruction *insn)
   } else if (p->vars[insn->src_args[1]].vartype == ORC_VAR_TYPE_PARAM) {
     /* FIXME this is a gross hack to reload the register with a
      * 64-bit version of the parameter. */
-    orc_x86_emit_mov_memoffset_sse (p, 4, 
+    orc_x86_emit_mov_memoffset_sse (p, 4,
         (int)ORC_STRUCT_OFFSET(OrcExecutor, params[insn->src_args[1]]),
         p->exec_reg, p->tmpreg, FALSE);
 
@@ -315,55 +313,49 @@ sse_rule_shift (OrcCompiler *p, void *user, OrcInstruction *insn)
 static void
 sse_rule_convsbw (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
-  orc_sse_emit_660f (p, "punpcklbw", 0x60,
-      p->vars[insn->src_args[0]].alloc,
-      p->vars[insn->dest_args[0]].alloc);
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest = p->vars[insn->dest_args[0]].alloc;
 
-  orc_sse_emit_shiftimm (p, "psraw", 0x71, 4, 8,
-      p->vars[insn->dest_args[0]].alloc);
+  orc_sse_emit_punpcklbw (p, src, dest);
+  orc_sse_emit_psraw (p, 8, dest);
 }
 
 static void
 sse_rule_convubw (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest = p->vars[insn->dest_args[0]].alloc;
+  int tmp = p->tmpreg;
+
   /* FIXME need a zero register */
-
   if (0) {
-    orc_sse_emit_660f (p, "punpcklbw", 0x60,
-        p->vars[insn->src_args[0]].alloc,
-        p->vars[insn->dest_args[0]].alloc);
-
-    orc_sse_emit_shiftimm (p, "psrlw", 0x71, 2, 8,
-        p->vars[insn->dest_args[0]].alloc);
+    orc_sse_emit_punpcklbw (p, src, dest);
+    orc_sse_emit_psrlw (p, 8, dest);
   } else {
-    if (p->vars[insn->src_args[0]].alloc !=
-        p->vars[insn->dest_args[0]].alloc) {
-      orc_sse_emit_660f (p, "movdqa", 0x6f,
-          p->vars[insn->src_args[0]].alloc,
-          p->vars[insn->dest_args[0]].alloc);
+    if (src != dest) {
+      orc_sse_emit_movdqa (p, src, dest);
     }
-    orc_sse_emit_660f (p, "pxor", 0xef,
-        p->tmpreg, p->tmpreg);
-    orc_sse_emit_660f (p, "punpcklbw", 0x60,
-        p->tmpreg,
-        p->vars[insn->dest_args[0]].alloc);
+    orc_sse_emit_pxor(p, tmp, tmp);
+    orc_sse_emit_punpcklbw (p, tmp, dest);
   }
 }
 
 static void
 sse_rule_convssswb (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
-  orc_sse_emit_660f (p, "packsswb", 0x63,
-      p->vars[insn->src_args[0]].alloc,
-      p->vars[insn->dest_args[0]].alloc);
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest = p->vars[insn->dest_args[0]].alloc;
+
+  orc_sse_emit_packsswb (p, src, dest);
 }
 
 static void
 sse_rule_convsuswb (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
-  orc_sse_emit_660f (p, "packuswb", 0x67,
-      p->vars[insn->src_args[0]].alloc,
-      p->vars[insn->dest_args[0]].alloc);
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest = p->vars[insn->dest_args[0]].alloc;
+
+  orc_sse_emit_packuswb (p, src, dest);
 }
 
 static void
@@ -375,52 +367,41 @@ sse_rule_convwb (OrcCompiler *p, void *user, OrcInstruction *insn)
   /* FIXME slow */
 
   if (dest != src) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_shiftimm (p, "psllw", 0x71, 6, 8,
-      p->vars[insn->dest_args[0]].alloc);
-  orc_sse_emit_shiftimm (p, "psrlw", 0x71, 2, 8,
-      p->vars[insn->dest_args[0]].alloc);
-  orc_sse_emit_660f (p, "packuswb", 0x67,
-      p->vars[insn->dest_args[0]].alloc,
-      p->vars[insn->dest_args[0]].alloc);
+  orc_sse_emit_psllw (p, 8, dest);
+  orc_sse_emit_psrlw (p, 8, dest);
+  orc_sse_emit_packuswb (p, dest, dest);
 }
 
 static void
 sse_rule_convswl (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
-  orc_sse_emit_660f (p, "punpcklwd", 0x61,
-      p->vars[insn->src_args[0]].alloc,
-      p->vars[insn->dest_args[0]].alloc);
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest = p->vars[insn->dest_args[0]].alloc;
 
-  orc_sse_emit_shiftimm (p, "psrad", 0x72, 4, 16,
-      p->vars[insn->dest_args[0]].alloc);
+  orc_sse_emit_punpcklwd (p, src, dest);
+  orc_sse_emit_psrad (p, 16, dest);
 }
 
 static void
 sse_rule_convuwl (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
-  /* FIXME need a zero register */
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest = p->vars[insn->dest_args[0]].alloc;
+  int tmp = p->tmpreg;
 
+  /* FIXME need a zero register */
   if (0) {
-    orc_sse_emit_660f (p, "punpcklwd", 0x61,
-        p->vars[insn->src_args[0]].alloc,
-        p->vars[insn->dest_args[0]].alloc);
-    orc_sse_emit_shiftimm (p, "psrld", 0x72, 2, 16,
-        p->vars[insn->dest_args[0]].alloc);
+    orc_sse_emit_punpcklwd (p, src, dest);
+    orc_sse_emit_psrld (p, 16, dest);
   } else {
-    if (p->vars[insn->src_args[0]].alloc !=
-        p->vars[insn->dest_args[0]].alloc) {
-      orc_sse_emit_660f (p, "movdqa", 0x6f,
-          p->vars[insn->src_args[0]].alloc,
-          p->vars[insn->dest_args[0]].alloc);
+    if (src != dest) {
+      orc_sse_emit_movdqa (p, src, dest);
     }
-    orc_sse_emit_660f (p, "pxor", 0xef,
-        p->tmpreg, p->tmpreg);
-    orc_sse_emit_660f (p, "punpcklwd", 0x61,
-        p->tmpreg,
-        p->vars[insn->dest_args[0]].alloc);
+    orc_sse_emit_pxor(p, tmp, tmp);
+    orc_sse_emit_punpcklwd (p, tmp, dest);
   }
 }
 
@@ -433,30 +414,30 @@ sse_rule_convlw (OrcCompiler *p, void *user, OrcInstruction *insn)
   /* FIXME slow */
 
   if (dest != src) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_shiftimm (p, "pslld", 0x72, 6, 16,
-      p->vars[insn->dest_args[0]].alloc);
-  orc_sse_emit_shiftimm (p, "psrad", 0x72, 4, 16,
-      p->vars[insn->dest_args[0]].alloc);
-  orc_sse_emit_660f (p, "packssdw", 0x6b, dest, dest);
+  orc_sse_emit_pslld (p, 16, dest);
+  orc_sse_emit_psrad (p, 16, dest);
+  orc_sse_emit_packssdw (p, dest, dest);
 }
 
 static void
 sse_rule_convssslw (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
-  orc_sse_emit_660f (p, "packssdw", 0x6b,
-      p->vars[insn->src_args[0]].alloc,
-      p->vars[insn->dest_args[0]].alloc);
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest = p->vars[insn->dest_args[0]].alloc;
+
+  orc_sse_emit_packssdw (p, src, dest);
 }
 
 static void
 sse_rule_convsuslw (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
-  orc_sse_emit_660f (p, "packusdw", 0x382b,
-      p->vars[insn->src_args[0]].alloc,
-      p->vars[insn->dest_args[0]].alloc);
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest = p->vars[insn->dest_args[0]].alloc;
+
+  orc_sse_emit_packusdw (p, src, dest);
 }
 
 static void
@@ -466,11 +447,11 @@ sse_rule_mulsbw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
 
-  orc_sse_emit_660f (p, "punpcklbw", 0x60, src, tmp);
-  orc_sse_emit_shiftimm (p, "psraw", 0x71, 4, 8, tmp);
-  orc_sse_emit_660f (p, "punpcklbw", 0x60, dest, dest);
-  orc_sse_emit_shiftimm (p, "psraw", 0x71, 4, 8, dest);
-  orc_sse_emit_660f (p, "pmullw", 0xd5, tmp, dest);
+  orc_sse_emit_punpcklbw (p, src, tmp);
+  orc_sse_emit_psraw (p, 8, tmp);
+  orc_sse_emit_punpcklbw (p, dest, dest);
+  orc_sse_emit_psraw (p, 8, dest);
+  orc_sse_emit_pmullw (p, tmp, dest);
 }
 
 static void
@@ -480,11 +461,11 @@ sse_rule_mulubw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
 
-  orc_sse_emit_660f (p, "punpcklbw", 0x60, src, tmp);
-  orc_sse_emit_shiftimm (p, "psrlw", 0x71, 2, 8, tmp);
-  orc_sse_emit_660f (p, "punpcklbw", 0x60, dest, dest);
-  orc_sse_emit_shiftimm (p, "psrlw", 0x71, 2, 8, dest);
-  orc_sse_emit_660f (p, "pmullw", 0xd5, tmp, dest);
+  orc_sse_emit_punpcklbw (p, src, tmp);
+  orc_sse_emit_psrlw (p, 8, tmp);
+  orc_sse_emit_punpcklbw (p, dest, dest);
+  orc_sse_emit_psrlw (p, 8, dest);
+  orc_sse_emit_pmullw (p, tmp, dest);
 }
 
 static void
@@ -494,10 +475,10 @@ sse_rule_mulswl (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
 
-  orc_sse_emit_660f (p, "movdqa", 0x6f, dest, tmp);
-  orc_sse_emit_660f (p, "pmulhw", 0xe5, src, tmp);
-  orc_sse_emit_660f (p, "pmullw", 0xd5, src, dest);
-  orc_sse_emit_660f (p, "punpcklwd", 0x61, tmp, dest);
+  orc_sse_emit_movdqa (p, dest, tmp);
+  orc_sse_emit_pmulhw (p, src, tmp);
+  orc_sse_emit_pmullw (p, src, dest);
+  orc_sse_emit_punpcklwd (p, tmp, dest);
 }
 
 static void
@@ -510,12 +491,12 @@ sse_rule_select0lw (OrcCompiler *p, void *user, OrcInstruction *insn)
   /* same as convlw */
 
   if (dest != src) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_shiftimm (p, "pslld", 0x72, 6, 16, dest);
-  orc_sse_emit_shiftimm (p, "psrad", 0x72, 4, 16, dest);
-  orc_sse_emit_660f (p, "packssdw", 0x6b, dest, dest);
+  orc_sse_emit_pslld (p, 16, dest);
+  orc_sse_emit_psrad (p, 16, dest);
+  orc_sse_emit_packssdw (p, dest, dest);
 }
 
 static void
@@ -527,11 +508,11 @@ sse_rule_select1lw (OrcCompiler *p, void *user, OrcInstruction *insn)
   /* FIXME slow */
 
   if (dest != src) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_shiftimm (p, "psrad", 0x72, 4, 16, dest);
-  orc_sse_emit_660f (p, "packssdw", 0x6b, dest, dest);
+  orc_sse_emit_psrad (p, 16, dest);
+  orc_sse_emit_packssdw (p, dest, dest);
 }
 
 static void
@@ -544,12 +525,12 @@ sse_rule_select0wb (OrcCompiler *p, void *user, OrcInstruction *insn)
   /* same as convwb */
 
   if (dest != src) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_shiftimm (p, "psllw", 0x71, 6, 8, dest);
-  orc_sse_emit_shiftimm (p, "psraw", 0x71, 4, 8, dest);
-  orc_sse_emit_660f (p, "packsswb", 0x63, dest, dest);
+  orc_sse_emit_psllw (p, 8, dest);
+  orc_sse_emit_psraw (p, 8, dest);
+  orc_sse_emit_packsswb (p, dest, dest);
 }
 
 static void
@@ -561,11 +542,60 @@ sse_rule_select1wb (OrcCompiler *p, void *user, OrcInstruction *insn)
   /* FIXME slow */
 
   if (dest != src) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_shiftimm (p, "psraw", 0x71, 4, 8, dest);
-  orc_sse_emit_660f (p, "packsswb", 0x63, dest, dest);
+  orc_sse_emit_psraw (p, 8, dest);
+  orc_sse_emit_packsswb (p, dest, dest);
+}
+
+static void
+sse_rule_splitlw (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest1 = p->vars[insn->dest_args[0]].alloc;
+  int dest2 = p->vars[insn->dest_args[1]].alloc;
+
+  /* FIXME slow */
+
+  if (dest1 != src) {
+    orc_sse_emit_movdqa (p, src, dest1);
+  }
+  orc_sse_emit_psrad (p, 16, dest1);
+  orc_sse_emit_packssdw (p, dest1, dest1);
+
+  if (dest2 != src) {
+    orc_sse_emit_movdqa (p, src, dest2);
+  }
+  orc_sse_emit_pslld (p, 16, dest2);
+  orc_sse_emit_psrad (p, 16, dest2);
+  orc_sse_emit_packssdw (p, dest2, dest2);
+
+}
+
+static void
+sse_rule_splitwb (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest1 = p->vars[insn->dest_args[0]].alloc;
+  int dest2 = p->vars[insn->dest_args[1]].alloc;
+
+  /* FIXME slow */
+
+  if (dest1 != src) {
+    orc_sse_emit_movdqa (p, src, dest1);
+  }
+
+  orc_sse_emit_psraw (p, 8, dest1);
+  orc_sse_emit_packsswb (p, dest1, dest1);
+
+  if (dest2 != src) {
+    orc_sse_emit_movdqa (p, src, dest2);
+  }
+
+  orc_sse_emit_psllw (p, 8, dest2);
+  orc_sse_emit_psraw (p, 8, dest2);
+  orc_sse_emit_packsswb (p, dest2, dest2);
 }
 
 static void
@@ -574,7 +604,7 @@ sse_rule_mergebw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[1]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
 
-  orc_sse_emit_660f (p, "punpcklbw", 0x60, src, dest);
+  orc_sse_emit_punpcklbw (p, src, dest);
 }
 
 static void
@@ -583,23 +613,24 @@ sse_rule_mergewl (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[1]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
 
-  orc_sse_emit_660f (p, "punpcklwd", 0x61, src, dest);
+  orc_sse_emit_punpcklwd (p, src, dest);
 }
 
 static void
 sse_emit_load_mask (OrcCompiler *p, unsigned int mask1, unsigned int mask2)
 {
   int tmp = p->tmpreg;
+  int gptmp = p->gp_tmpreg;
   int tmp2 = X86_XMM7;
 
-  orc_x86_emit_mov_imm_reg (p, 4, mask1, p->gp_tmpreg);
-  orc_x86_emit_mov_reg_sse (p, p->gp_tmpreg, p->tmpreg);
+  orc_x86_emit_mov_imm_reg (p, 4, mask1, gptmp);
+  orc_x86_emit_mov_reg_sse (p, gptmp, tmp);
   orc_sse_emit_pshufd (p, 0, tmp, tmp);
-  orc_x86_emit_mov_imm_reg (p, 4, mask2, p->gp_tmpreg);
-  orc_x86_emit_mov_reg_sse (p, p->gp_tmpreg, tmp2);
-  orc_sse_emit_660f (p, "punpcklbw", 0x60, tmp2, tmp2);
-  orc_sse_emit_660f (p, "punpcklwd", 0x61, tmp2, tmp2);
-  orc_sse_emit_660f (p, "paddb", 0xfc, tmp2, tmp);
+  orc_x86_emit_mov_imm_reg (p, 4, mask2, gptmp);
+  orc_x86_emit_mov_reg_sse (p, gptmp, tmp2);
+  orc_sse_emit_punpcklbw (p, tmp2, tmp2);
+  orc_sse_emit_punpcklwd (p, tmp2, tmp2);
+  orc_sse_emit_paddb (p, tmp2, tmp);
 }
 
 static void
@@ -608,16 +639,16 @@ sse_rule_select0lw_sse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
-  
+
   /* FIXME slow */
 
   sse_emit_load_mask (p, 0x05040100, 0x08000800);
 
   if (src != dest) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_660f (p, "pshufb", 0x3800, tmp, dest);
+  orc_sse_emit_pshufb (p, tmp, dest);
 }
 
 static void
@@ -626,16 +657,16 @@ sse_rule_select1lw_sse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
-  
+
   /* FIXME slow */
 
   sse_emit_load_mask (p, 0x07060302, 0x08000800);
 
   if (src != dest) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_660f (p, "pshufb", 0x3800, tmp, dest);
+  orc_sse_emit_pshufb (p, tmp, dest);
 }
 
 static void
@@ -644,16 +675,16 @@ sse_rule_select0wb_sse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
-  
+
   /* FIXME slow */
 
   sse_emit_load_mask (p, 0x06040200, 0x08000800);
 
   if (src != dest) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_660f (p, "pshufb", 0x3800, tmp, dest);
+  orc_sse_emit_pshufb (p, tmp, dest);
 }
 
 static void
@@ -662,16 +693,16 @@ sse_rule_select1wb_sse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
-  
+
   /* FIXME slow */
 
   sse_emit_load_mask (p, 0x07050301, 0x08000800);
 
   if (src != dest) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
 
-  orc_sse_emit_660f (p, "pshufb", 0x3800, tmp, dest);
+  orc_sse_emit_pshufb (p, tmp, dest);
 }
 
 /* slow rules */
@@ -682,15 +713,15 @@ sse_rule_swapw_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
-  
+
   /* FIXME slow */
 
   sse_emit_load_mask (p, 0x02030001, 0x0c080400);
 
   if (src != dest) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
-  orc_sse_emit_660f (p, "pshufb", 0x3800, tmp, dest);
+  orc_sse_emit_pshufb (p, tmp, dest);
 }
 
 static void
@@ -699,15 +730,15 @@ sse_rule_swapl_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[0]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
-  
+
   /* FIXME slow */
 
   sse_emit_load_mask (p, 0x00010203, 0x0c080400);
 
   if (src != dest) {
-    orc_sse_emit_660f (p, "movdqa", 0x6f, src, dest);
+    orc_sse_emit_movdqa (p, src, dest);
   }
-  orc_sse_emit_660f (p, "pshufb", 0x3800, tmp, dest);
+  orc_sse_emit_pshufb (p, tmp, dest);
 }
 
 static void
@@ -716,15 +747,16 @@ sse_rule_maxuw_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
   int src = p->vars[insn->src_args[1]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
+  int gptmp = p->gp_tmpreg;
 
-  orc_x86_emit_mov_imm_reg (p, 4, 0x80008000, p->gp_tmpreg);
-  orc_x86_emit_mov_reg_sse (p, p->gp_tmpreg, tmp);
+  orc_x86_emit_mov_imm_reg (p, 4, 0x80008000, gptmp);
+  orc_x86_emit_mov_reg_sse (p, gptmp, tmp);
   orc_sse_emit_pshufd (p, 0, tmp, tmp);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
-  orc_sse_emit_660f (p, "pmaxsw", 0xee, src, dest);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
+  orc_sse_emit_pmaxsw (p, src, dest);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
 }
 
 static void
@@ -732,15 +764,15 @@ sse_rule_minuw_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src = p->vars[insn->src_args[1]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
-  int tmp = p->tmpreg;
+  int tmp;
 
-  orc_sse_emit_loadiw (p, tmp, 0x8000);
+  tmp = orc_compiler_get_constant (p, 2, 0x8000);
 
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
-  orc_sse_emit_660f (p, "pminsw", 0xea, src, dest);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
+  orc_sse_emit_pminsw (p, src, dest);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
 }
 
 static void
@@ -748,15 +780,15 @@ sse_rule_avgsb_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src = p->vars[insn->src_args[1]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
-  int tmp = p->tmpreg;
+  int tmp;
 
-  orc_sse_emit_loadib (p, tmp, 0x80);
+  tmp = orc_compiler_get_constant (p, 1, 0x80);
 
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
-  orc_sse_emit_660f (p, "pavgb", 0xe0, src, dest);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
+  orc_sse_emit_pavgb (p, src, dest);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
 }
 
 static void
@@ -764,15 +796,15 @@ sse_rule_avgsw_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src = p->vars[insn->src_args[1]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
-  int tmp = p->tmpreg;
+  int tmp;
 
-  orc_sse_emit_loadiw (p, tmp, 0x8000);
+  tmp = orc_compiler_get_constant (p, 2, 0x8000);
 
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
-  orc_sse_emit_660f (p, "pavgw", 0xe3, src, dest);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
+  orc_sse_emit_pavgw (p, src, dest);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
 }
 
 static void
@@ -782,11 +814,11 @@ sse_rule_maxsb_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
 
-  orc_sse_emit_660f (p, "movdqa", 0x6f, dest, tmp);
-  orc_sse_emit_660f (p, "pcmpgtb", 0x64, src, tmp);
-  orc_sse_emit_660f (p, "pand", 0xdb, tmp, dest);
-  orc_sse_emit_660f (p, "pandn", 0xdf, src, tmp);
-  orc_sse_emit_660f (p, "por", 0xeb, tmp, dest);
+  orc_sse_emit_movdqa (p, dest, tmp);
+  orc_sse_emit_pcmpgtb (p, src, tmp);
+  orc_sse_emit_pand (p, tmp, dest);
+  orc_sse_emit_pandn (p, src, tmp);
+  orc_sse_emit_por (p, tmp, dest);
 }
 
 static void
@@ -796,11 +828,11 @@ sse_rule_minsb_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
 
-  orc_sse_emit_660f (p, "movdqa", 0x6f, src, tmp);
-  orc_sse_emit_660f (p, "pcmpgtb", 0x64, dest, tmp);
-  orc_sse_emit_660f (p, "pand", 0xdb, tmp, dest);
-  orc_sse_emit_660f (p, "pandn", 0xdf, src, tmp);
-  orc_sse_emit_660f (p, "por", 0xeb, tmp, dest);
+  orc_sse_emit_movdqa (p, src, tmp);
+  orc_sse_emit_pcmpgtb (p, dest, tmp);
+  orc_sse_emit_pand (p, tmp, dest);
+  orc_sse_emit_pandn (p, src, tmp);
+  orc_sse_emit_por (p, tmp, dest);
 }
 
 static void
@@ -810,11 +842,11 @@ sse_rule_maxsl_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
 
-  orc_sse_emit_660f (p, "movdqa", 0x6f, dest, tmp);
-  orc_sse_emit_660f (p, "pcmpgtd", 0x66, src, tmp);
-  orc_sse_emit_660f (p, "pand", 0xdb, tmp, dest);
-  orc_sse_emit_660f (p, "pandn", 0xdf, src, tmp);
-  orc_sse_emit_660f (p, "por", 0xeb, tmp, dest);
+  orc_sse_emit_movdqa (p, dest, tmp);
+  orc_sse_emit_pcmpgtd (p, src, tmp);
+  orc_sse_emit_pand (p, tmp, dest);
+  orc_sse_emit_pandn (p, src, tmp);
+  orc_sse_emit_por (p, tmp, dest);
 }
 
 static void
@@ -824,11 +856,11 @@ sse_rule_minsl_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = p->vars[insn->dest_args[0]].alloc;
   int tmp = p->tmpreg;
 
-  orc_sse_emit_660f (p, "movdqa", 0x6f, src, tmp);
-  orc_sse_emit_660f (p, "pcmpgtd", 0x66, dest, tmp);
-  orc_sse_emit_660f (p, "pand", 0xdb, tmp, dest);
-  orc_sse_emit_660f (p, "pandn", 0xdf, src, tmp);
-  orc_sse_emit_660f (p, "por", 0xeb, tmp, dest);
+  orc_sse_emit_movdqa (p, src, tmp);
+  orc_sse_emit_pcmpgtd (p, dest, tmp);
+  orc_sse_emit_pand (p, tmp, dest);
+  orc_sse_emit_pandn (p, src, tmp);
+  orc_sse_emit_por (p, tmp, dest);
 }
 
 static void
@@ -836,21 +868,21 @@ sse_rule_maxul_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src = p->vars[insn->src_args[1]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
-  int tmp = p->tmpreg;
+  int tmp;
 
-  orc_sse_emit_loadil (p, tmp, 0x80000000);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
+  tmp = orc_compiler_get_constant (p, 4, 0x80000000);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
 
-  orc_sse_emit_660f (p, "movdqa", 0x6f, dest, tmp);
-  orc_sse_emit_660f (p, "pcmpgtd", 0x66, src, tmp);
-  orc_sse_emit_660f (p, "pand", 0xdb, tmp, dest);
-  orc_sse_emit_660f (p, "pandn", 0xdf, src, tmp);
-  orc_sse_emit_660f (p, "por", 0xeb, tmp, dest);
+  orc_sse_emit_movdqa (p, dest, tmp);
+  orc_sse_emit_pcmpgtd (p, src, tmp);
+  orc_sse_emit_pand (p, tmp, dest);
+  orc_sse_emit_pandn (p, src, tmp);
+  orc_sse_emit_por (p, tmp, dest);
 
-  orc_sse_emit_loadil (p, tmp, 0x80000000);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
+  tmp = orc_compiler_get_constant (p, 4, 0x80000000);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
 }
 
 static void
@@ -858,23 +890,93 @@ sse_rule_minul_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src = p->vars[insn->src_args[1]].alloc;
   int dest = p->vars[insn->dest_args[0]].alloc;
-  int tmp = p->tmpreg;
+  int tmp;
 
-  orc_sse_emit_loadil (p, tmp, 0x80000000);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
+  tmp = orc_compiler_get_constant (p, 4, 0x80000000);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
 
-  orc_sse_emit_660f (p, "movdqa", 0x6f, src, tmp);
-  orc_sse_emit_660f (p, "pcmpgtd", 0x66, dest, tmp);
-  orc_sse_emit_660f (p, "pand", 0xdb, tmp, dest);
-  orc_sse_emit_660f (p, "pandn", 0xdf, src, tmp);
-  orc_sse_emit_660f (p, "por", 0xeb, tmp, dest);
+  orc_sse_emit_movdqa (p, src, tmp);
+  orc_sse_emit_pcmpgtd (p, dest, tmp);
+  orc_sse_emit_pand (p, tmp, dest);
+  orc_sse_emit_pandn (p, src, tmp);
+  orc_sse_emit_por (p, tmp, dest);
 
-  orc_sse_emit_loadil (p, tmp, 0x80000000);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, src);
-  orc_sse_emit_660f (p, "pxor", 0xef, tmp, dest);
+  tmp = orc_compiler_get_constant (p, 4, 0x80000000);
+  orc_sse_emit_pxor(p, tmp, src);
+  orc_sse_emit_pxor(p, tmp, dest);
 }
 
+/* float ops */
+
+#define UNARY_F(opcode,insn_name,code) \
+static void \
+sse_rule_ ## opcode (OrcCompiler *p, void *user, OrcInstruction *insn) \
+{ \
+  orc_sse_emit_0f (p, insn_name, code, \
+      p->vars[insn->src_args[0]].alloc, \
+      p->vars[insn->dest_args[0]].alloc); \
+}
+
+#define BINARY_F(opcode,insn_name,code) \
+static void \
+sse_rule_ ## opcode (OrcCompiler *p, void *user, OrcInstruction *insn) \
+{ \
+  orc_sse_emit_0f (p, insn_name, code, \
+      p->vars[insn->src_args[1]].alloc, \
+      p->vars[insn->dest_args[0]].alloc); \
+}
+
+BINARY_F(addf, "addps", 0x58)
+BINARY_F(subf, "subps", 0x5c)
+BINARY_F(mulf, "mulps", 0x59)
+BINARY_F(divf, "divps", 0x5e)
+BINARY_F(maxf, "maxps", 0x5f)
+BINARY_F(minf, "minps", 0x5d)
+UNARY_F(sqrtf, "sqrtps", 0x51)
+
+static void
+sse_rule_cmpeqf (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  orc_sse_emit_0f (p, "cmpeqps", 0xc2,
+      p->vars[insn->src_args[1]].alloc,
+      p->vars[insn->dest_args[0]].alloc);
+  *p->codeptr++ = 0x00;
+}
+
+static void
+sse_rule_cmpltf (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  orc_sse_emit_0f (p, "cmpltps", 0xc2,
+      p->vars[insn->src_args[1]].alloc,
+      p->vars[insn->dest_args[0]].alloc);
+  *p->codeptr++ = 0x01;
+}
+
+static void
+sse_rule_cmplef (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  orc_sse_emit_0f (p, "cmpleps", 0xc2,
+      p->vars[insn->src_args[1]].alloc,
+      p->vars[insn->dest_args[0]].alloc);
+  *p->codeptr++ = 0x02;
+}
+
+static void
+sse_rule_convfl (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  orc_sse_emit_660f (p, "cvtps2dq", 0x5b,
+      p->vars[insn->src_args[0]].alloc,
+      p->vars[insn->dest_args[0]].alloc);
+}
+
+static void
+sse_rule_convlf (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  orc_sse_emit_0f (p, "cvtdq2ps", 0x5b,
+      p->vars[insn->src_args[0]].alloc,
+      p->vars[insn->dest_args[0]].alloc);
+}
 
 void
 orc_compiler_sse_register_rules (OrcTarget *target)
@@ -968,6 +1070,19 @@ orc_compiler_sse_register_rules (OrcTarget *target)
   orc_rule_register (rule_set, "accl", sse_rule_accl, NULL);
   orc_rule_register (rule_set, "accsadubl", sse_rule_accsadubl, NULL);
 
+  orc_rule_register (rule_set, "addf", sse_rule_addf, NULL);
+  orc_rule_register (rule_set, "subf", sse_rule_subf, NULL);
+  orc_rule_register (rule_set, "mulf", sse_rule_mulf, NULL);
+  orc_rule_register (rule_set, "divf", sse_rule_divf, NULL);
+  orc_rule_register (rule_set, "minf", sse_rule_minf, NULL);
+  orc_rule_register (rule_set, "maxf", sse_rule_maxf, NULL);
+  orc_rule_register (rule_set, "sqrtf", sse_rule_sqrtf, NULL);
+  orc_rule_register (rule_set, "cmpeqf", sse_rule_cmpeqf, NULL);
+  orc_rule_register (rule_set, "cmpltf", sse_rule_cmpltf, NULL);
+  orc_rule_register (rule_set, "cmplef", sse_rule_cmplef, NULL);
+  orc_rule_register (rule_set, "convfl", sse_rule_convfl, NULL);
+  orc_rule_register (rule_set, "convlf", sse_rule_convlf, NULL);
+
   /* slow rules */
   orc_rule_register (rule_set, "maxuw", sse_rule_maxuw_slow, NULL);
   orc_rule_register (rule_set, "minuw", sse_rule_minuw_slow, NULL);
@@ -1001,6 +1116,8 @@ orc_compiler_sse_register_rules (OrcTarget *target)
   orc_rule_register (rule_set, "select1lw", sse_rule_select1lw_sse3, NULL);
   orc_rule_register (rule_set, "select0wb", sse_rule_select0wb_sse3, NULL);
   orc_rule_register (rule_set, "select1wb", sse_rule_select1wb_sse3, NULL);
+  orc_rule_register (rule_set, "splitlw", sse_rule_splitlw, NULL);
+  orc_rule_register (rule_set, "splitwb", sse_rule_splitwb, NULL);
 
   /* SSE 4.1 */
   rule_set = orc_rule_set_new (orc_opcode_set_get("sys"), target,
